@@ -1,3 +1,6 @@
+# Copyright (C) 2021, QuantStack
+# SPDX-License-Identifier: BSD-3-Clause
+
 from boa.core.solver import get_solver
 import copy
 import re
@@ -8,21 +11,21 @@ import sys
 from dataclasses import dataclass
 from typing import Tuple
 
-from rich.console import Console
+import rich
 from rich.table import Table
 from rich.padding import Padding
 
 
 # TODO remove
-import conda_build.jinja_context
 from conda.base.context import context
 from conda_build.config import get_or_merge_config
 from conda_build.utils import apply_pin_expressions
 from conda.models.channel import Channel as CondaChannel
-from conda.core.package_cache_data import PackageCacheData
 from conda_build.metadata import eval_selector, ns_cfg
 
-console = Console()
+from boa.core.config import boa_config
+
+console = boa_config.console
 
 
 @dataclass
@@ -85,7 +88,7 @@ class CondaBuildSpec:
         return self.final
 
     def eval_pin_subpackage(self, all_outputs):
-        if not self.splitted[1].startswith("PIN_SUBPACKAGE"):
+        if len(self.splitted) <= 1 or not self.splitted[1].startswith("PIN_SUBPACKAGE"):
             return
         pkg_name = self.name
         max_pin, exact = self.splitted[1][len("PIN_SUBPACKAGE") + 1 : -1].split(",")
@@ -112,6 +115,8 @@ class CondaBuildSpec:
             self.final = f"{pkg_name} {version_pin}"
 
     def eval_pin_compatible(self, build, host):
+        if len(self.splitted) <= 1:
+            return
 
         lower_bound, upper_bound, min_pin, max_pin, exact = self.splitted[1][
             len("PIN_COMPATIBLE") + 1 : -1
@@ -164,6 +169,7 @@ class Output:
         self.version = d["package"]["version"]
         self.build_string = d["package"].get("build_string")
         self.build_number = d["build"].get("number", 0)
+        self.noarch = d["build"].get("noarch", False)
         self.is_first = False
         self.sections = {}
 
@@ -229,6 +235,7 @@ class Output:
             self.requirements[section] = [
                 CondaBuildSpec(r) for r in (self.requirements.get(section) or [])
             ]
+        self.sections["requirements"] = self.requirements
 
         # handle strong and weak run exports
         run_exports = []
@@ -246,11 +253,17 @@ class Output:
             self.sections["build"]["run_exports"] = run_exports
 
     def skip(self):
-        skips = self.sections["build"].get("skip")
-
-        if skips:
-            return any([eval_selector(x, ns_cfg(self.config), []) for x in skips])
-        return False
+        skips = self.sections["build"].get("skip", [])
+        skip_reasons = []
+        for x in skips:
+            if eval_selector(x, ns_cfg(self.config), []):
+                skip_reasons.append(x)
+        if len(skip_reasons):
+            console.print(
+                f"[green]Skipping {self.name} {' | '.join(self.differentiating_variant)} because of[/green]\n"
+                + "\n".join([f"  - {x}" for x in skip_reasons])
+            )
+        return len(skip_reasons) != 0
 
     def all_requirements(self):
         requirements = (
@@ -280,19 +293,22 @@ class Output:
                 copied.requirements["host"][idx].from_pinnings = True
 
         # todo figure out if we should pin like that in the run reqs as well?
-        for idx, r in enumerate(self.requirements["run"]):
-            vname = r.name.replace("-", "_")
-            if vname in variant:
-                copied.requirements["run"][idx] = CondaBuildSpec(
-                    r.name + " " + variant[vname]
-                )
-                copied.requirements["run"][idx].from_pinnings = True
+        # for idx, r in enumerate(self.requirements["run"]):
+        #     vname = r.name.replace("-", "_")
+        #     if vname in variant:
+        #         copied.requirements["run"][idx] = CondaBuildSpec(
+        #             r.name + " " + variant[vname]
+        #         )
+        #         copied.requirements["run"][idx].from_pinnings = True
 
         # insert compiler_cxx, compiler_c and compiler_fortran
         for idx, r in enumerate(self.requirements["build"]):
             if r.name.startswith("COMPILER_"):
                 lang = r.splitted[1].lower()
-                compiler = conda_build.jinja_context.compiler(lang, self.config)
+                if variant.get(lang + "_compiler"):
+                    compiler = (
+                        f"{variant[lang + '_compiler']}_{variant['target_platform']}"
+                    )
                 if variant.get(lang + "_compiler_version"):
                     version = variant[lang + "_compiler_version"]
                     copied.requirements["build"][idx].final = f"{compiler} {version}*"
@@ -312,10 +328,51 @@ class Output:
 
         return copied
 
-    def __rich__(self):
-        from rich import box
+    def to_json(self):
+        res = {
+            "name": self.name,
+            "version": self.version,
+            "build_number": self.build_number,
+            "source": self.data["source"],
+            "noarch": self.noarch,
+        }
 
-        table = Table(box=box.MINIMAL_DOUBLE_HEAD)
+        res["differentiating_variant"] = self.differentiating_variant
+        res["variant"] = self.variant
+
+        def specs_to_dict(specs):
+            ret = []
+            for s in specs:
+                attrs = []
+                if s.is_pin:
+                    attrs.append("is_pin")
+                if s.from_run_export:
+                    attrs.append("from_run_export")
+                if s.from_pinnings:
+                    attrs.append("from_pinnings")
+
+                sdict = {
+                    "spec": s.raw,
+                    "name": s.final_name,
+                    "attrs": attrs,
+                    "resolved": {},
+                }
+                if hasattr(s, "final_version"):
+                    sdict["resolved"]["final_version"] = s.final_version
+
+                ret.append(sdict)
+            return ret
+
+        res["requirements"] = {
+            "build": specs_to_dict(self.requirements["build"]),
+            "host": specs_to_dict(self.requirements["host"]),
+            "run": specs_to_dict(self.requirements["run"]),
+        }
+
+        return res
+
+    def __rich__(self):
+        table = Table(box=rich.box.MINIMAL_DOUBLE_HEAD)
         s = f"Output: {self.name} {self.version} BN: {self.build_number}\n"
         if hasattr(self, "differentiating_variant"):
             short_v = " ".join([val for val in self.differentiating_variant])
@@ -396,6 +453,10 @@ class Output:
             add_header("Run", True)
             for r in self.requirements["run"]:
                 spec_format(r)
+        if self.requirements["run_constrained"]:
+            add_header("Run Constraints", True)
+            for r in self.requirements["run_constrained"]:
+                spec_format(r)
         return table
 
     def __repr__(self):
@@ -490,21 +551,28 @@ class Output:
 
         if env == "build":
             for rex in collected_run_exports:
-                if "strong" in rex:
-                    for r in rex["strong"]:
+                if not self.noarch:
+                    for r in rex.get("strong", []):
                         append_or_replace("host", r)
                         append_or_replace("run", r)
-                if "weak" in rex:
-                    for r in rex["weak"]:
+                    for r in rex.get("weak", []):
                         append_or_replace("host", r)
+                    for r in rex.get("strong_constrains", []):
+                        append_or_replace("run_constrained", r)
 
         if env == "host":
             for rex in collected_run_exports:
-                if "strong" in rex:
-                    for r in rex["strong"]:
+                if not self.noarch:
+                    for r in rex.get("strong", []):
                         append_or_replace("run", r)
-                if "weak" in rex:
-                    for r in rex["weak"]:
+                    for r in rex.get("weak", []):
+                        append_or_replace("run", r)
+                    for r in rex.get("strong_constrains", []):
+                        append_or_replace("run_constrained", r)
+                    for r in rex.get("weak_constrains", []):
+                        append_or_replace("run_constrained", r)
+                else:
+                    for r in rex.get("noarch", []):
                         append_or_replace("run", r)
 
     def _solve_env(self, env, all_outputs):
@@ -525,7 +593,6 @@ class Output:
             spec_map = {s.final_name: s for s in specs}
             specs = [str(x) for x in specs]
 
-            pkg_cache = PackageCacheData.first_writable().pkgs_dir
             if env in ("host", "run") and not self.config.subdirs_same:
                 subdir = self.config.host_subdir
             else:
@@ -556,7 +623,7 @@ class Output:
             }
 
             downloaded = t.fetch_extract_packages(
-                pkg_cache, solver.repos + list(solver.local_repos.values()),
+                solver.repos + list(solver.local_repos.values()),
             )
             if not downloaded:
                 raise RuntimeError("Did not succeed in downloading packages.")
